function targ = set_gettarget(eeg)
% Target = set_gettarget(EEG)
% Generic function to extract the target values from an EEG set; works for both epoched and continuous data sets.
%
% In:
%   EEG     : EEGLAB dataset, either epoched or continuous
%
% Out:
%   Target  : target values for the dataset; either per epoch or per sample
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-04-07

% make sure that the data is evaluated
eeg = exp_eval(eeg,1);

if isfield(eeg,'epoch') && ~isempty(eeg.epoch)
    % epoched EEG dataset: get per-epoch targets
    if ~isfield(eeg.epoch,'target')
        error('no ''target'' field in the given epochs'); end
    targ = vertcat(eeg.epoch.target);
elseif isfield(eeg,'chanlocs') && isfield(eeg.chanlocs,'type') && size(eeg.data,3) == 1
    % continuous EEG dataset: get per-sample epoch targets
    targchans = strcmp('target',{eeg.chanlocs.type});
    if ~any(targchans)
        error('no ''target'' channels available in the given data.'); end
    targ = eeg.data(targchans,:);
else
    error('set_gettarget needs an epoch field for epoched data or a channel type field for continuous data.');
end
