function [result,idxmap] = set_merge(varargin)
% [Merged,Index-Map] = set_merge(Set-#1, Set-#2, ...)
% Merge epoched EEGLAB datasets.
%
% In:
%   Set-#k   : dataset #k
%
% Out:
%   Merged : the merged epoched eeg set
%   Index-Map : a mapping from trial index (in the merged set), to dataset index 
%               (in the list of sets supplied)
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-03-31


idxmap = [];
if ~exp_begindef('editing') return; end

% create the index map & identify non-empty sets
sets = {};
for i=1:length(varargin)
    if isfield(varargin{i},'data') 
        sets{end+1} = varargin{i};
        idxmap = [idxmap i*ones(1,size(varargin{i}.data,3))]; 
    end
end

% do the merging
result = sets{1};
for i=2:length(sets)
    result = pop_mergeset(result,sets{i}); end

exp_enddef;
