function [exp,changed,rules] = exp_replaceall(exp,rules)
% [Out-Expression,Changed,Remaining-Rules] = exp_replaceall(In-Expression, Rule/Rule-List)
% apply substitution rules to In-Expression and return the result in Out-Expression; 
% each part of the expression receives at most a single replacement, and each rule is used at most once
% rules may contain blanks and placeholders.
%
% In:
%   In-Expression : some pure expression in which substitution shall take place
%   Rule-List     : cell array of substitution rules
%
% Out:
%   Out-Expression     : In-Expression with substitutions performed
%
%   Changed            : whether a replacement has taken place
%   Remaining-Rules    : the list of rules that have not been applied
%
% Notes:
%   Impure expressions degrade into pure expressions, when substitutions are performed on them (i.e. they lose their attached value).
%   Named patterns inside the right-hand-sides of substitution rules are treated as if just their symbols were given instead.
%
%                                       Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                                       2010-04-19

if ~exp_begindef('symbolic') return; end

[exp,changed,rules] = utl_replaceall(exp,rules);

exp_enddef;