function [exp,waschanged] = exp_replacerepeated(exp,rules)
% apply the given list of substitution rules to the expression until the expression no longer changes; 
% rules may contain blanks and placeholders.
%
% In:
%   In-Expression : some expression in which substitution shall take place
%   Rule-List     : cell array of substutition rules; may also be a struct, where field names are taken as symbol names
%
% Out:
%   Out-Expression     : In-Expression with substitutions performed
%   Changed            : whether a replacement has taken place
%
% Notes:
%   Impure expressions degrade into pure expressions when substitutions are performed on them (i.e. they lose their value).
%
%                                       Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                                       2010-04-19

if ~exp_begindef('symbolic') return; end

[exp,waschanged] = utl_replacerepeated(exp,rules);

exp_enddef;