function eeg = flt_project(eeg, mat, newchans)
% Out-Set = flt_project(In-Set, Matrix, Channel-Names)
% Spatially project the given dataset, e.g. to apply an IC decomposition
%
% In:
%   In-Set : epoched or continuous EEGLAB data set
%
%   Matrix : projection matrix
%
%   Channel-Names : optional cell array of new channel names (default: {'1','2','3',...})
%   
%                           Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                           2010-03-28

if ~exp_begindef('filter') return; end;

[C,S,T] = size(eeg.data);
if ~exist('mat','var') || isempty(mat) mat = eye(C); end
if ~exist('newchans','var') newchans = []; end

% project data
eeg.data = reshape(mat*reshape(eeg.data,C,[]),[],S,T);
eeg.nbchan = size(eeg.data,1);

% rewrite chanlocs
if isempty(newchans)
    eeg.chanlocs = struct('labels',cellfun(@num2str,num2cell(1:eeg.nbchan,1),'UniformOutput',false));
elseif length(newchans) == eeg.nbchan
    if isfield(newchans,'labels')
        eeg.chanlocs = newchans;
    elseif iscellstr(newchans)
        eeg.chanlocs = pop_chanedit(struct('labels',newchans),'lookup',[]);        
    elseif isnumeric(newchans)
        eeg.chanlocs = eeg.chanlocs(newchans);
    else
        error('unsupported channel format');
    end
else
    error('channel length does not match projected data channel count');
end

exp_enddef;