function eeg = flt_reref(eeg, chn, varargin)
% Usage: Result-Set = flt_reref(EEG, Channels, key/vals)
% Re-references the data to a new (set of) channel(s) or the average of all channels.
%
% Re-referencing is a spatial filter in which the (instantaneous) mean signal of some "reference" channels is subtracted from the 
% signal of each channel. This allows to dampen externally-induced signals (e.g. electromagnetic interference) which are captured by all sensors, including
% those that are references. A frequently used variant is the average reference, in which the mean of all channels is subtracted from every channel.
% Dedicated reference channels are usually behind the ears ("mastoids"), on the nose bone ("nasion") or sometimes on the scalp, whereas this cancels brain signals
% out of nearby channels. Many BCI paradigms do not require an explicit re-referencing, since their spatial filters are adaptively optimized and effectively incorporate
% re-referencing to the degree that it is necessary.
%
% In:
%   EEG      :  Epoched or Continuous eeg data set.
%
%   Channels :  Cell array of reference channels to which the eeg data should be referenced, or [] for average reference. (default: [])
%
%   keys/vals:  Same as in pop_reref.
%
% Out:
%   Result-Set : Re-referenced eeg data set.
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-03-28

% sort names/values
varargin = hlp_sort_namevalues(varargin{:});
if ~exp_begindef('filter') return; end
if isempty(chn)
    chn = []; end

eeg = pop_reref(eeg,set_chanid(eeg,chn),varargin{:});

exp_enddef;