function outargs = hlp_lookup_symbol(x,context)
% Internal. Perform a lookup for a symbol in the current (dynamic) scope formed by (possibly nested) exp_block(s)
% returns a cell array of all the values associated with the symbol
% Used to implement dynamic scoping across functions, used by parts of the expression system.

global bcilab;

if ~exist('context','var')
    % obtain the execution context by throwing an exception
    % ... this is relatively slow, since the context must be compiled into a MATLAB struct
    try 
        throw; %#ok<LTARG> fastest way to get an exception
    catch
        context = lasterror;  %#ok<LERR>
    end
end

% get all stackframe entries in the context
entries = {context.stack.name};
framepos = strfind(entries,'active_stackframe__');
% and look for our symbol in the corresponding stack frames
symbol = char(x);
for i=find(~cellfun('isempty',framepos))
    frameid = entries{i}((framepos{i}+19):end-12);
    % ...to look up the symbol in question
    if isfield(bcilab.stackframes.ov.(frameid),symbol)
        outargs = {bcilab.stackframes.ov.(frameid).(symbol)};
        return;
    end
end
if isfield(bcilab.workspace.ov,symbol)
    % symbol is assigned in the global workspace
    outargs = {bcilab.workspace.ov.(symbol)};
else
    % symbol remains unsubstituted
    outargs = {x};
end
