function args = hlp_struct2varargin(struc,varargin)
% Args = hlp_struct2varargin(Struct,Options...)
% convert a struct (not a struct array) into a seqence of name-value pairs; inverts the operation of hlp_varargin2struct.
%
% In:
%   Struct: a 1x1 structure array
%   Options: optional name-value pairs;
%            'restrict': restrict the output names to those listed in the following cell array
%            'suppress': suppress the output names listed in the following cell array
%
% Out:
%  Args: a list of name-value pairs
%
% Examples:
%  hlp_struct2varargin(options,'restrict',{'test','arg1','arg2'});
%  hlp_struct2varargin(options,'suppress',{'myarg1','myarg2'});
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-03-28

opts = hlp_varargin2struct(varargin,'restrict',[],'suppress',[]);

fields = fieldnames(struc)';
data = struct2cell(struc)'; 
if ~isempty(opts.restrict)
    [fields,I,J] = intersect(fields,opts.restrict);  %#ok<NASGU>
    data = data(I);
end
if ~isempty(opts.suppress)
    [fields,I] = setdiff(fields,opts.suppress);
    data = data(I);
end

args = vertcat(fields,data); 
args = args(:)';