function pred = ml_predictdal(trials,model)
% predict the target probability for some test set, according to dual-augmented lagrangian model
%
% In:
%   Trials  : the data a matrix, as in ml_predict
%
%   Model   : predictive model as produced by ml_traindal
%
% Out:
%   Prediction  : discrete probability distribution, formatted as
%                 {'disc' [NxC] [Cx1]}, with element #2 being the per-class probability and 
%                 element #3 the original target values per class
%                 thus, the expected target values are Prediction{2}*Prediction{3}
%
% See also:
%   ml_traindal()
%
%                           Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                           2010-06-25

if isfield(model,'voted')
    % dispatch to the voter
    pred = ml_predictvote(trials,model);
else
    % vectorize trials if necessary
    if ndims(trials) == 3
        trials = double(reshape(trials,[],size(trials,3))'); end

    % scale data
    trials = hlp_applyscaling(trials,model.sc_info);

    try
    model.w(end+1) = model.b;
    catch
        keyboard
    end
    
    % transpose if necessary
    if model.transpose
        trials = double(reshape(trials',model.opts.shape(1),model.opts.shape(2),[]));
        ntrials = zeros(model.opts.shape(2),model.opts.shape(1),size(trials,3));
        for t=1:size(trials,3)
            ntrials(:,:,t) = trials(:,:,t)'; end
        trials = double(reshape(ntrials,[],size(ntrials,3))');
    end
    
    % add bias term
    trials = [trials ones(size(trials,1),1)];
    
    % map to probabilities
    probs = 1 ./ (1 + exp(-trials*full(model.w)));
    pred = {'disc', [1-probs probs], model.classes};
end