function result = para_modulators(data, mode, varargin)
% Result = para_modulators(Input-Data, Operation-Mode, Options...)
% Paradigm for oscillatory processes using the Independent Modulators approach roughly similar to [1]. 
% Note: this is experimental test code which is not ready for public consumption. Requires an Independent 
% Modulators data structure given as the 'modulators' parameter.
%
% In:
%       Input-Set       : some data set
%
%       Operation-Mode  : 'preprocessing', to pre-process the Input-Set according to the paradigm (and parameters)
%                         'learning', to learn a predictive model from a pre-processed Input-Set (and parameters)
%                         'prediction', to make predictions given a pre-processed Input-Set and a Model
%
%       User Options... : optional parameters of the paradigm; name-value pairs 
%                         --- for preprocessing (see flt_default)
%                         * 'epoch':  epoch window, in seconds relative to event (default: [-1.5 1.5])
%                         * 'events':  event types to determine epochs and the target variable (user-specified)
%                         * others, by default turned off: see flt_default
%                         --- for learning
%                         * 'modulators': modulators control structure, generated by Nima
%                         * 'learner': learning function to be used on the features; (default: 'rvm')
%                         --- for prediction
%                         * 'model': the detector model (as produced during the learning)
% Out:
%       Result : depends on the Operation-Mode; 
%                * for 'preprocessing', this is the preprocessed dataset
%                * for 'learning', this is the learned model
%                * for 'prediction', this is the predictions produced by the model, given the data
%
% References:
%   [1] Onton J & Makeig S. "Broadband high-frequency EEG dynamics during emotion imagination."
%       Frontiers in Human Neuroscience, 2009. 
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-04-29

result = para_skeleton(data, mode, hlp_varargin2struct(varargin, ...
	... % preprocessing defaults:
    'epoch',[-1 1], 'modulators',[], ...
    ... % learning defaults:
    'learner','rvm', 'featureadapt',@(D,opts)opts.modulators, ...
    'featureextract',@calculate_im_and_ic_spectral_activations_on_new_data_for_bci));