function vis_lineplot(subjs,arr)
% vis_lineplot(Subjects, All-Results)
% draws the line plots
% In:
%       Subjects        :   array containing the subject's indices
%       Data-Struct     :   struct array containing one struct for each drawn plot line
%                           required fields:
%                            - all  : as it comes from bench_averages
%                            - path : cell array describing the path from each subject's structure to the displayed information
%                                     may contain strings (to enter fields) and numbers (to index cell arrays)
%                            - col  : color of the line, 3d vector of the form [Red Green Blue] ( 0 <= color value <= 1)
%                            - name : name of the field; for the legend
%
% Example:
%       % load the data
%       [csplda,csplda_avg] = bench_averages('/store/data/bench_results/LDA/cspreal_',13:55);
%       [speclda,speclda_avg] = bench_averages('/store/data/bench_results/LDA/specreal_',13:55);
%       
%       % generate the data description, here just two data sets
%       erd(1) = struct('all',{csplda},'path',{{'analysis',4,'nownd','cverr'}},'col',[1 0 0],'name','CSP-LDA');
%       erd(2) = struct('all',{speclda},'path',{{'analysis',4,'nownd','cverr'}},'col',[0 1 0],'name','specCSP-LDA');
%
%       % plot a line plot, using a selection of the subjects
%       figure; vis_lineplot([13:37 43:55],erd);
%
%       % plot a diag plot, using a selection of the subjects
%       figure; vis_diagplot([13:37 43:55],erd);
%
%                                               Christian Kothe, FG MMS, TU-Berlin
%                                               13.04.2007


hold;
% arr is a structure array consisting of the data array (in arr(i).all) and the path to the values (in arr(i).path)
if isstruct(arr) && length(arr) < 8
    % draw multiple lines
    for i=1:length(arr)
        [h(i),n{i}] = drawlines(arr(i),subjs);
    end
    legend(h,n{:});
end
hold;


function [handle,name] = drawlines(arr,subjs,color)

    if iscell(arr.all)    
        % we have to extract the pts from the data
        k=1;
        for i=subjs
            try
                cur = arr.all{i};
                for j=1:length(arr.path)
                    if ischar(arr.path{j})
                        cur = cur.(arr.path{j});
                    elseif isnumeric(arr.path{j})
                        cur = cur{arr.path{j}};
                    end
                end
                data(k) = cur;
            catch
                data(k) = data(k-1);
            end
            k = k+1;        
        end
    else
        % ... a numerical array -- NYI
    end

    % now draw the data
    if isfield(arr,'col') color = arr.col; else color = [0 0 1]; end
    if isfield(arr,'name') name = arr.name; else name = 'unspec'; end    
    line([0 43], [0.5 0],'color',[ 1 1 1]);
    for x = 1:length(data)-1
        handle = line([x x+1],[data(x) data(x+1)],'color',color,'LineWidth',2);
    end
