function res = utl_default_partitioner(data,inds,unused) %#ok<INUSD>
% the default partitioner for data in cross-validations; supports generic data structures, agnostic of EEG data set, etc.
%   if only data and no inds are given, return the number of available indices
%   otherwise return data restricted to the given index set

if isnumeric(data)
    % partition along the highest dimension (i.e. 2nd or 3rd)
    if ~exist('inds','var') || isempty(inds)
        siz = size(data);
        res = siz(end);
    else
        if ndims(data) == 2
            res = data(:,inds);
        elseif ndims(data) == 3
            res = data(:,:,inds);
        else
            error('undetermined partitioning scheme for the given data');
        end
    end
elseif isstruct(data) && length(data) > 1
    if ~exist('inds','var') || isempty(inds) 
        res = length(data);
    else
        res = data(inds);
    end
elseif iscell(data) && length(data) == 2 && isnumeric(data{1}) && isnumeric(data{2}) ...
        && size(data{1},1) == size(data{2},1) && size(data{1},2) ~= 1 && size(data{2},2) == 1
    % a cell array of the form {NxF,Nx1} -- indicating data and target variable; partitioned along N; F>1
    if ~exist('inds','var') || isempty(inds)
        res = length(data{2});
    else
        res = {data{1}(inds,:),data{2}(inds,:)};
    end
elseif iscell(data) && length(data) == 2 && isnumeric(data{1}) && iscell(data{2}) && length(data{2}) > 1 && ischar(data{2}{1}) ...
        && ((isnumeric(data{2}{2}) && size(data{1},1) == size(data{2}{2},1)) || (iscell(data{2}{2}) && size(data{1},1) == length(data{2}{2})))
    % a cell array of the form {NxF,{distrib,[NxD],...}} or {NxF,{distrib,{N},...}} --  indicating data and target variable, partitioned along N
    if ~exist('inds','var') || isempty(inds)
        res = size(data{1},1);
    else
        if isnumeric(data{2}{2})
            res = {data{1}(inds,:), {data{2}{1},data{2}{2}(inds,:), data{2}{3:end}}};
        else
            res = {data{1}(inds,:), {data{2}{1},data{2}{2}(inds), data{2}{3:end}}};
        end
    end
elseif iscell(data)
    % regular cell array: partition along the linear indices of the cell array
    if ~exist('inds','var') || isempty(inds)
        res = length(data);
    else
        res = data(inds);
    end
elseif isstruct(data) && isfield(data,'X') && isfield(data,'Y') && size(data.X,1) == size(data.Y,1)
    % structure with X&Y fields: partition along the first dimension of X&Y
    if ~exist('inds','var') || isempty(inds)
        res = size(data.X,1);
    else
        res = data;
        res.X = res.X(inds,:);
        res.Y = res.Y(inds,:);
    end
else
    error('unsupported data format for cross-validation; please specify a custom data partitioner.');
end
end
