function t = utl_default_target(data)
% the default target extraction function for cross-validations
% supports {Data,Target} for Target in any format that can be generated by ml_predict

if iscell(data) && length(data) == 2 && isnumeric(data{1}) && isnumeric(data{2}) && size(data{1},1) == size(data{2},1)
    % we have a regression target inside a {NxF,NxT} data format
    t = data{2};
elseif iscell(data) && length(data) == 2 && iscell(data{2}) && length(data{2}) > 1 && ischar(data{2}{1})
    % we have a distribution target inside a {NxF,{distrib,...} data format
    t = data{2};
elseif isstruct(data) && isfield(data,'Y')
    % we have a struct with fields X&Y
    t = data.Y;
else
    error('unknown target variable in the supplied data; please specify a custom target extractor.');
end
end