function Y = utl_picktimes(X,wnd)
% Result = utl_picktimes(Input, Windows)
% averages input array within the given sample ranges, for each trial and channel
%
% In:
%   Input   : [Channels x Samples x Trials] data array
%   Windows : [Ranges x 2] array specifying the beginning and end of each range, 
%             in samples (floating-point numbers are allowed)
%
% Out:
%   Result  : resulting data array of dimensions [Channels x Ranges x Trials]
%
%                               Christian Kothe, Swartz Center for Computational Neuroscience, UCSD
%                               2010-04-20

% determine the data layout
[C,S,T] = size(X);

% fix the windows, if necessary
wnd = min(max(wnd,1),S);
wnd(:,2) = wnd(:,2)+1;
wnd = wnd(wnd(:,2)>wnd(:,1),:);
W = size(wnd,1);

% calc offset & coverage values for first & last sample
fo = floor(wnd(:,1));
fc = min(fo+1,wnd(:,2)) - wnd(:,1);
lo = ceil(wnd(:,2)-1);
lc = wnd(:,2) - max(lo,wnd(:,1));
% calc fully covered sample range, inverse length
for r=1:W
    full{r} = fo(r)+1:lo(r)-1; end
ilen = 1./(fc+cellfun(@length,full)'+lc); 

Y = zeros(C,W,T);
% accumulate for every range...
for r=1:W    
    Y(:,r,:) = (X(:,fo(r),:)*fc(r)+sum(X(:,full{r},:),2)+X(:,lo(r),:)*lc(r))*ilen(r); end
